/****************************************************************************
*   Dino - simple file manager
*   Copyright (C) 2012  Dino project

* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.

* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.

* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/
#include "ditemmodel.h"
#include <QtGui>



ditemmodel::ditemmodel(const QFileInfo& fileInfo, ditemmodel* parent)
{
    mParent = parent;
    mFileInfo = fileInfo;
    walked = false;
    dirty = false;

    if(parent)
    {
          parent->addChild(this);
          mAbsFilePath = fileInfo.filePath();
    }
    else
    {
        walked = true;
        mAbsFilePath = "";
    }
}


ditemmodel::~ditemmodel()
{
    qDeleteAll(mChildren);
}
ditemmodel* ditemmodel::childAt(int position)
{
    return mChildren.value(position,0);
}
int ditemmodel::childCount() const
{
    if(walked) return mChildren.count();
    return 1;
}
bool ditemmodel::hasChild(QString fileName)
{
    foreach(ditemmodel * item, mChildren)
        if(item->fileName() == fileName) return true;

    return false;
}
int ditemmodel::childNumber() const
{
    if(mParent)
    {
      return mParent->mChildren.indexOf(const_cast<ditemmodel*>(this));
    }

    return 0;
}
QList<ditemmodel*> ditemmodel::children()
{
    return mChildren;
}

//---------------------------------------------------------------------------------------
ditemmodel* ditemmodel::parent()
{
    return mParent;
}

//---------------------------------------------------------------------------------------
QString ditemmodel::absoluteFilePath()const
{
    return mAbsFilePath;
}

//---------------------------------------------------------------------------------------
QString ditemmodel::fileName() const
{
    if(mAbsFilePath == "/") return "/";
    else return mFileInfo.fileName();

}
QFileInfo ditemmodel::fileInfo() const
{
    return mFileInfo;
}
void ditemmodel::refreshFileInfo()
{
    mFileInfo.refresh();
    mPermissions.clear();
}
void ditemmodel::addChild(ditemmodel *child)
{
    if(!mChildren.contains(child))
    {
      mChildren.append(child);
    }
}
void ditemmodel::removeChild(ditemmodel *child)
{
    mChildren.removeOne(child);
    delete child;
}
void ditemmodel::clearAll()
{
    foreach(ditemmodel *child, mChildren)
        delete child;
    mChildren.clear();
    walked = 0;
}
void ditemmodel::changeName(QString newName)
{
    mAbsFilePath = mParent->absoluteFilePath() + QDir::separator() + newName;
    mFileInfo.setFile(mAbsFilePath);
    clearAll();
}
ditemmodel* ditemmodel::matchPath(const QStringList& path, int startIndex)
{
    QStringList temp = path;
    temp.replace(0,"/");
    temp.removeAll("");

    if(walked == 0)     //not populated yet
    {
        walked = true;
        QDir dir(this->absoluteFilePath());
        QFileInfoList all = dir.entryInfoList(QDir::AllEntries | QDir::NoDotAndDotDot | QDir::Hidden | QDir::System);

        foreach(QFileInfo one, all)
            new ditemmodel(one,this);
    }

    foreach(ditemmodel* child, mChildren)
    {
        QString match = temp.at(startIndex);

        if(child->fileName() == match)
        {
            if(startIndex + 1 == temp.count()) return child;
            else return child->matchPath(path,startIndex + 1);
        }
    }

    return 0;
}




